package ru.csasq.cit_is_bot

import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.NavigationBar
import androidx.compose.material3.NavigationBarItem
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Snackbar
import androidx.compose.material3.SnackbarHost
import androidx.compose.material3.SnackbarHostState
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBar
import androidx.compose.runtime.*
import androidx.compose.ui.text.style.TextAlign
import androidx.navigation.NavDestination.Companion.hierarchy
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.currentBackStackEntryAsState
import androidx.navigation.compose.rememberNavController
import cit_is_bot.composeapp.generated.resources.Res
import cit_is_bot.composeapp.generated.resources.baseline_chat_24
import cit_is_bot.composeapp.generated.resources.baseline_dashboard_24
import cit_is_bot.composeapp.generated.resources.baseline_edit_calendar_24
import cit_is_bot.composeapp.generated.resources.baseline_supervisor_account_24
import org.jetbrains.compose.resources.DrawableResource
import org.jetbrains.compose.resources.painterResource
import ru.csasq.cit_is_bot.ui.screens.ChatsScreen
import ru.csasq.cit_is_bot.ui.screens.DashboardScreen
import ru.csasq.cit_is_bot.ui.screens.ScenariosScreen
import ru.csasq.cit_is_bot.ui.screens.UsersScreen
import ru.csasq.cit_is_bot.ui.theme.darkScheme

sealed class Screen(
    val route: String,
    val title: String,
    val icon: DrawableResource,
) {
    data object Dashboard : Screen(
        route = "dashboard",
        title = "Дашборд",
        icon = Res.drawable.baseline_dashboard_24,
    )
    data object Scenarios : Screen(
        route = "scenarios",
        title = "Сценарии",
        icon = Res.drawable.baseline_edit_calendar_24,
    )
    data object Chats : Screen(
        route = "chats",
        title = "Чаты",
        icon = Res.drawable.baseline_chat_24,
    )
    data object Users : Screen(
        route = "users",
        title = "Пользователи",
        icon = Res.drawable.baseline_supervisor_account_24,
    )
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun App() {
    MaterialTheme(
        colorScheme = darkScheme,
    ) {
        val navController = rememberNavController()
        val snackbarHostState = remember {
            SnackbarHostState()
        }
        val floatingActionButtonState = remember {
            mutableStateOf<(@Composable () -> Unit)?>(null)
        }

        Scaffold(
            topBar = {
                TopAppBar(
                    title = {
                        Text(
                            text = "Сообщения",
                        )
                    },
                )
            },
            bottomBar = {
                NavigationBar {
                    val navBackStackEntry = navController.currentBackStackEntryAsState()
                    val items = listOf(
                        Screen.Dashboard,
                        Screen.Scenarios,
                        Screen.Chats,
                        Screen.Users,
                    )
                    items.forEach { screen ->
                        NavigationBarItem(
                            icon = {
                                Icon(
                                    painter = painterResource(screen.icon),
                                    contentDescription = null,
                                )
                            },
                            label = {
                                Text(
                                    text = screen.title,
                                    textAlign = TextAlign.Center,
                                )
                            },
                            selected = navBackStackEntry.value?.destination?.hierarchy?.any {
                                it.route == screen.route
                            } == true,
                            onClick = {
                                floatingActionButtonState.value = null
                                navController.navigate(screen.route)
                            },
                        )
                    }
                }
            },
            snackbarHost = {
                SnackbarHost(
                    hostState = snackbarHostState,
                ) {
                    Snackbar(
                        snackbarData = it,
                        containerColor = MaterialTheme.colorScheme.errorContainer,
                        contentColor = MaterialTheme.colorScheme.error,
                        actionColor = MaterialTheme.colorScheme.onErrorContainer,
                    )
                }
            },
            floatingActionButton = {
                floatingActionButtonState.value?.let {
                    it()
                }
            },
        ) {
            paddingValues -> NavHost(
                navController = navController,
                startDestination = Screen.Dashboard.route,
            ) {
                composable(Screen.Dashboard.route) {
                    DashboardScreen(navController, paddingValues)
                }
                composable(Screen.Scenarios.route) {
                    ScenariosScreen(
                        navController,
                        paddingValues,
                        floatingActionButtonState,
                    )
                }
                composable(Screen.Chats.route) {
                    ChatsScreen(navController, paddingValues)
                }
                composable(Screen.Users.route) {
                    UsersScreen(navController, paddingValues)
                }
            }
        }
    }
}